"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicServiceClient = exports.Cache = void 0;
const ServiceWizard_1 = require("../coreServices/ServiceWizard");
const GenericClient_1 = require("./GenericClient");
class Cache {
    constructor({ itemLifetime, monitoringFrequency, waiterTimeout, waiterFrequency }) {
        this.cache = new Map();
        this.cacheLifetime = itemLifetime || 1800000;
        this.monitoringFrequency = monitoringFrequency || 60000;
        this.waiterTimeout = waiterTimeout || 30000;
        this.waiterFrequency = waiterFrequency || 100;
        this.isMonitoring = false;
    }
    runMonitor() {
        if (this.isMonitoring) {
            return;
        }
        this.isMonitoring = true;
        setTimeout(() => {
            const newCache = new Map();
            let cacheRenewed = false;
            Object.keys(this.cache).forEach((id) => {
                const item = this.cache.get(id);
                if (!this.isExpired(item)) {
                    newCache.set(id, item);
                    cacheRenewed = true;
                }
            });
            this.cache = newCache;
            this.isMonitoring = false;
            if (cacheRenewed) {
                this.runMonitor();
            }
        }, this.monitoringFrequency);
    }
    isExpired(cacheItem) {
        const now = new Date().getTime();
        const elapsed = now - cacheItem.createdAt;
        return elapsed > this.cacheLifetime;
    }
    getItem(id) {
        if (this.cache.get(id) === undefined) {
            return null;
        }
        const cached = this.cache.get(id);
        if (this.isExpired(cached)) {
            this.cache.delete(id);
            return;
        }
        return cached;
    }
    reserveWaiter(id, fetcher) {
        return new Promise((resolve, reject) => {
            const started = new Date().getTime();
            const waiting = true;
            const waiter = () => {
                if (!waiting) {
                    return;
                }
                setTimeout(() => {
                    const item = this.cache.get(id);
                    if (!item) {
                        this.reserveAndFetch(id, fetcher)
                            .then(() => {
                            resolve(this.cache.get(id));
                        })
                            .catch((err) => {
                            reject(err);
                        });
                    }
                    else if (item.reserved) {
                        const elapsed = new Date().getTime() - started;
                        if (elapsed < this.waiterTimeout) {
                            waiter();
                            return;
                        }
                        else {
                            reject(new Error('Timed-out waiting for cache item to become available; timeout ' +
                                this.waiterTimeout +
                                ', waited ' +
                                elapsed));
                            return;
                        }
                    }
                    else {
                        resolve(item);
                    }
                }, this.waiterFrequency);
            };
            waiter();
        });
    }
    reserveAndFetch(id, fetcher) {
        this.reserveItem(id, fetcher);
        const fetchPromise = fetcher()
            .then((result) => {
            this.setItem(id, result, fetcher);
            return result;
        })
            .finally(() => {
        });
        return fetchPromise;
    }
    getItemWithWait({ id, fetcher }) {
        const cached = this.cache.get(id);
        if (cached) {
            if (this.isExpired(cached)) {
                this.cache.delete(id);
            }
            else if (cached.reserved) {
                return this.reserveWaiter(id, fetcher).then((cached) => {
                    return cached.value;
                });
            }
            else {
                return Promise.resolve(cached.value);
            }
        }
        return this.reserveAndFetch(id, fetcher);
    }
    reserveItem(id, fetcher) {
        this.cache.set(id, {
            id: id,
            createdAt: new Date().getTime(),
            reserved: true,
            value: null,
            fetcher: fetcher
        });
    }
    setItem(id, value, fetcher) {
        if (this.cache.has(id)) {
            const item = this.cache.get(id);
            if (item && item.reserved) {
                item.reserved = false;
                item.value = value;
                item.fetcher = fetcher;
            }
            else {
                this.cache.set(id, {
                    id: id,
                    createdAt: new Date().getTime(),
                    fetcher: fetcher,
                    reserved: false,
                    value: value
                });
            }
        }
        else {
            this.cache.set(id, {
                id: id,
                createdAt: new Date().getTime(),
                fetcher: fetcher,
                reserved: false,
                value: value
            });
        }
        this.runMonitor();
    }
}
exports.Cache = Cache;
var moduleCache = new Cache({
    itemLifetime: 1800000,
    monitoringFrequency: 60000,
    waiterTimeout: 30000,
    waiterFrequency: 100
});
const DEFAULT_TIMEOUT = 10000;
class DynamicServiceClient {
    constructor({ token, url, version, timeout, rpcContext }) {
        this.token = token;
        this.timeout = timeout || DEFAULT_TIMEOUT;
        this.rpcContext = rpcContext || null;
        if (!url) {
            throw new Error('The service discovery url was not provided');
        }
        this.url = url;
        this.version = version || null;
        if (version === 'auto') {
            this.version = null;
        }
    }
    options() {
        return {
            timeout: this.timeout,
            authorization: this.token,
            rpcContext: this.rpcContext
        };
    }
    getModule() {
        return this.constructor.module;
    }
    moduleId() {
        let moduleId;
        if (!this.version) {
            moduleId = this.getModule() + ':auto';
        }
        else {
            moduleId = this.getModule() + ':' + this.version;
        }
        return moduleId;
    }
    getCached(fetcher) {
        return moduleCache.getItemWithWait({
            id: this.moduleId(),
            fetcher: fetcher
        });
    }
    lookupModule() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getCached(() => {
                const client = new ServiceWizard_1.ServiceWizardClient({
                    url: this.url,
                    token: this.token,
                    timeout: this.timeout
                });
                return Promise.resolve(client.getServiceStatus({
                    module_name: this.getModule(),
                    version: this.version
                }));
            });
        });
    }
    callFunc(funcName, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const moduleInfo = yield this.lookupModule();
            const client = new GenericClient_1.AuthorizedGenericClient({
                module: moduleInfo.module_name,
                url: moduleInfo.url,
                token: this.token
            });
            return yield client.callFunc(funcName, params);
        });
    }
}
exports.DynamicServiceClient = DynamicServiceClient;
//# sourceMappingURL=DynamicServiceClient.js.map