"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PubSubProxy = void 0;
const uuid_1 = require("uuid");
const SEND_WINDOW = 1000 / 60;
class PubSubProxy {
    constructor(pubsub) {
        this.subscriptions = [];
        this.pubsub = pubsub;
    }
    on(messageID, handler) {
        const id = this.pubsub.on(messageID, handler);
        this.subscriptions.push(id);
    }
    send(messageID, payload) {
        this.pubsub.send(messageID, payload);
    }
    off() {
        this.subscriptions.forEach((id) => {
            this.pubsub.off(id);
        });
    }
}
exports.PubSubProxy = PubSubProxy;
class PubSub {
    constructor() {
        this.sendQueue = [];
        this.messageListeners = new Map();
        this.allListeners = new Map();
        this.sendPending = false;
    }
    sendMessages() {
        const queue = this.sendQueue;
        this.sendQueue = [];
        queue.forEach((message) => {
            const messageListener = this.messageListeners.get(message.id);
            if (!messageListener) {
                return;
            }
            messageListener.listeners.forEach((listener) => {
                try {
                    listener.handler(message.payload);
                }
                catch (ex) {
                    if (listener.errorHandler) {
                        try {
                            listener.errorHandler(ex);
                        }
                        catch (ex) {
                            console.error('ERROR in error handler: ' + ex.message);
                        }
                    }
                    else {
                        console.error('ERROR not handled: ' + ex.message, ex);
                    }
                }
            });
        });
    }
    activateSend() {
        if (this.sendPending) {
            return;
        }
        this.sendPending = true;
        window.setTimeout(() => {
            this.sendMessages();
            this.sendPending = false;
        }, SEND_WINDOW);
    }
    send(messageID, payload) {
        const message = {
            id: messageID,
            payload
        };
        this.sendQueue.push(message);
        this.activateSend();
    }
    on(messageID, handler, errorHandler) {
        let messageListener = this.messageListeners.get(messageID);
        if (!messageListener) {
            messageListener = {
                listeners: []
            };
            this.messageListeners.set(messageID, messageListener);
        }
        const id = uuid_1.v4();
        const listener = {
            id, messageID, handler, errorHandler
        };
        messageListener.listeners.push(listener);
        this.allListeners.set(id, listener);
        return id;
    }
    off(id) {
        const listener = this.allListeners.get(id);
        if (!listener) {
            return;
        }
        const messageListener = this.messageListeners.get(listener.messageID);
        messageListener.listeners = messageListener.listeners.filter((l) => {
            return l.id !== listener.id;
        });
        this.allListeners.delete(id);
    }
}
exports.default = PubSub;
//# sourceMappingURL=PubSub.js.map